@@ -1,40 +1,47 @@
 """Rule definition for JSON Schema Validations."""
 from __future__ import annotations
 
-import json
 import logging
-import os
 import sys
-from functools import lru_cache
 from typing import Any
 
-import yaml
-from jsonschema import validate
-from jsonschema.exceptions import ValidationError
-
-from ansiblelint.config import JSON_SCHEMAS
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable
-from ansiblelint.loaders import yaml_load_safe
 from ansiblelint.rules import AnsibleLintRule
-from ansiblelint.schemas import __file__ as schemas_module
+from ansiblelint.schemas import JSON_SCHEMAS, validate_file_schema
 
 _logger = logging.getLogger(__name__)
 
+
 DESCRIPTION_MD = """ Returned errors will not include exact line numbers, but they will mention
-the schema name being used as a tag, like ``playbook-schema``,
-``tasks-schema``.
+the schema name being used as a tag, like ``schema[playbook]``,
+``schema[tasks]``.
 
 This rule is not skippable and stops further processing of the file.
 
-Schema bugs should be reported towards [schemas](https://github.com/ansible/schemas) project instead of ansible-lint.
-
 If incorrect schema was picked, you might want to either:
 
 * move the file to standard location, so its file is detected correctly.
 * use ``kinds:`` option in linter config to help it pick correct file type.
 """
 
+pre_checks = {
+    "task": {
+        "with_flattened": {
+            "msg": "with_flattened was moved to with_community.general.flattened in 2.10",
+            "tag": "moves",
+        },
+        "with_filetree": {
+            "msg": "with_filetree was moved to with_community.general.flattened in 2.10",
+            "tag": "moves",
+        },
+        "with_cartesian": {
+            "msg": "with_cartesian was moved to with_community.general.flattened in 2.10",
+            "tag": "moves",
+        },
+    }
+}
+
 
 class ValidateSchemaRule(AnsibleLintRule):
     """Perform JSON Schema Validation for known lintable kinds."""
@@ -46,13 +53,24 @@ class ValidateSchemaRule(AnsibleLintRule):
     tags = ["core", "experimental"]
     version_added = "v6.1.0"
 
-    @staticmethod
-    @lru_cache(maxsize=None)
-    def _get_schema(kind: str) -> Any:
-        """Return the schema for the given kind."""
-        schema_file = os.path.dirname(schemas_module) + "/" + kind + ".json"
-        with open(schema_file, encoding="utf-8") as f:
-            return json.load(f)
+    def matchtask(
+        self, task: dict[str, Any], file: Lintable | None = None
+    ) -> bool | str | MatchError | list[MatchError]:
+        result = []
+        for key in pre_checks["task"]:
+            if key in task:
+                msg = pre_checks["task"][key]["msg"]
+                tag = pre_checks["task"][key]["tag"]
+                result.append(
+                    MatchError(
+                        message=msg,
+                        filename=file,
+                        rule=ValidateSchemaRule(),
+                        details=ValidateSchemaRule.description,
+                        tag=f"schema[{tag}]",
+                    )
+                )
+        return result
 
     def matchyaml(self, file: Lintable) -> list[MatchError]:
         """Return JSON validation errors found as a list of MatchError(s)."""
@@ -60,23 +78,17 @@ class ValidateSchemaRule(AnsibleLintRule):
         if file.kind not in JSON_SCHEMAS:
             return []
 
-        try:
-            # convert yaml to json (keys are converted to strings)
-            yaml_data = yaml_load_safe(file.content)
-            json_data = json.loads(json.dumps(yaml_data))
-            validate(
-                instance=json_data,
-                schema=ValidateSchemaRule._get_schema(file.kind),
-            )
-        except yaml.constructor.ConstructorError:
-            _logger.debug(
-                "Ignored failure to load %s for schema validation, as !vault may cause it."
-            )
-            return []
-        except ValidationError as exc:
+        errors = validate_file_schema(file)
+        if errors:
+            if errors[0].startswith("Failed to load YAML file"):
+                _logger.debug(
+                    "Ignored failure to load %s for schema validation, as !vault may cause it."
+                )
+                return []
+
             result.append(
                 MatchError(
-                    message=exc.message,
+                    message=errors[0],
                     filename=file,
                     rule=ValidateSchemaRule(),
                     details=ValidateSchemaRule.description,
@@ -99,7 +111,7 @@ if "pytest" in sys.modules:
         ("file", "expected_kind", "expected"),
         (
             (
-                "examples/galaxy.yml",
+                "examples/collection/galaxy.yml",
                 "galaxy",
                 ["'GPL' is not one of"],
             ),
@@ -215,3 +227,31 @@ if "pytest" in sys.modules:
             assert result.filename.endswith(file)
             assert expected[idx] in result.message
             assert result.tag == f"schema[{expected_kind}]"
+
+    @pytest.mark.parametrize(
+        ("file", "expected_kind", "expected_tag", "count"),
+        (
+            pytest.param(
+                "examples/playbooks/rule-syntax-moves.yml",
+                "playbook",
+                "schema[moves]",
+                3,
+                id="playbook",
+            ),
+        ),
+    )
+    def test_schema_moves(
+        file: str, expected_kind: str, expected_tag: str, count: int
+    ) -> None:
+        """Validate ability to detect schema[moves]."""
+        lintable = Lintable(file)
+        assert lintable.kind == expected_kind
+
+        rules = RulesCollection(options=options)
+        rules.register(ValidateSchemaRule())
+        results = Runner(lintable, rules=rules).run()
+
+        assert len(results) == count, results
+        for result in results:
+            assert result.filename.endswith(file)
+            assert result.tag == expected_tag
