@@ -9,47 +9,16 @@ from typing import TYPE_CHECKING, Any
 from ansible.parsing.yaml.objects import AnsibleUnicode
 
 from ansiblelint.config import options
+from ansiblelint.constants import LINE_NUMBER_KEY, SUCCESS_RC
 from ansiblelint.file_utils import Lintable
-from ansiblelint.rules import AnsibleLintRule
+from ansiblelint.rules import AnsibleLintRule, RulesCollection
+from ansiblelint.runner import Runner
 from ansiblelint.skip_utils import get_rule_skips_from_line
-from ansiblelint.utils import LINE_NUMBER_KEY, parse_yaml_from_file
+from ansiblelint.utils import parse_yaml_from_file
 
 if TYPE_CHECKING:
-    from ansiblelint.constants import odict
     from ansiblelint.errors import MatchError
 
-
-# Should raise var-naming at line [4, 8, 18, 20].
-FAIL_PLAY = """---
-- hosts: localhost
-  vars:
-    CamelCaseIsBad: false  # invalid
-    this_is_valid:  # valid because content is a dict, not a variable
-      CamelCase: ...
-      ALL_CAPS: ...
-    ALL_CAPS_ARE_BAD_TOO: ...  # invalid
-    CamelCaseButErrorIgnored: true  # noqa: var-naming
-
-  tasks:
-    - name: Foo
-      ansible.builtin.set_fact:
-        "{{ 'test_' }}var": "value"  # valid
-    - name: Bar
-      ansible.builtin.set_fact:
-        CamelCaseButErrorIgnored: true  # noqa: var-naming
-    - name: Test in a block
-      block:
-        - name:
-          ansible.builtin.set_fact:
-            CamelCaseIsBad: "{{ BAD }}" # invalid
-          vars:
-            ALL_CAPS_ARE_BAD_TOO: "{{ MoreBad }}" # invalid
-      vars:
-        BAD: false # invalid
-        MoreBad: ... # invalid
-"""
-
-
 # Should raise var-naming at line [2, 6].
 FAIL_VARS = """---
 CamelCaseIsBad: false  # invalid
@@ -75,6 +44,7 @@ class VariableNamingRule(AnsibleLintRule):
     severity = "MEDIUM"
     tags = ["idiom", "experimental"]
     version_added = "v5.0.10"
+    needs_raw_task = True
     re_pattern = re.compile(options.var_naming_pattern or "^[a-z_][a-z0-9_]*$")
 
     def is_invalid_variable_name(self, ident: str) -> bool:
@@ -100,12 +70,12 @@ class VariableNamingRule(AnsibleLintRule):
         # safety measure.
         return not bool(self.re_pattern.match(ident))
 
-    def matchplay(self, file: Lintable, data: odict[str, Any]) -> list[MatchError]:
+    def matchplay(self, file: Lintable, data: dict[str, Any]) -> list[MatchError]:
         """Return matches found for a specific playbook."""
         results: list[MatchError] = []
         raw_results: list[MatchError] = []
 
-        if not data:
+        if not data or file.kind not in ("tasks", "handlers", "playbook", "vars"):
             return results
         # If the Play uses the 'vars' section to set variables
         our_vars = data.get("vars", {})
@@ -120,6 +90,7 @@ class VariableNamingRule(AnsibleLintRule):
                         message="Play defines variable '"
                         + key
                         + "' within 'vars' section that violates variable naming standards",
+                        tag=f"var-naming[{key}]",
                     )
                 )
         if raw_results:
@@ -134,28 +105,52 @@ class VariableNamingRule(AnsibleLintRule):
 
     def matchtask(
         self, task: dict[str, Any], file: Lintable | None = None
-    ) -> bool | str:
+    ) -> list[MatchError]:
         """Return matches for task based variables."""
+        results = []
         # If the task uses the 'vars' section to set variables
         our_vars = task.get("vars", {})
         for key in our_vars.keys():
             if self.is_invalid_variable_name(key):
-                return "Task defines variables within 'vars' section that violates variable naming standards"
+                results.append(
+                    self.create_matcherror(
+                        filename=file,
+                        linenumber=our_vars[LINE_NUMBER_KEY],
+                        message=f"Task defines variable within 'vars' section that violates variable naming standards: {key}",
+                        tag=f"var-naming[{key}]",
+                    )
+                )
 
         # If the task uses the 'set_fact' module
         ansible_module = task["action"]["__ansible_module__"]
-        ansible_action = task["action"]
         if ansible_module == "set_fact":
-            for key in ansible_action.keys():
+            for key in filter(
+                lambda x: isinstance(x, str) and not x.startswith("__"),
+                task["action"].keys(),
+            ):
                 if self.is_invalid_variable_name(key):
-                    return "Task uses 'set_fact' to define variables that violates variable naming standards"
+                    results.append(
+                        self.create_matcherror(
+                            filename=file,
+                            linenumber=task["action"][LINE_NUMBER_KEY],
+                            message=f"Task uses 'set_fact' to define variables that violates variable naming standards: {key}",
+                            tag=f"var-naming[{key}]",
+                        )
+                    )
 
         # If the task registers a variable
         registered_var = task.get("register", None)
         if registered_var and self.is_invalid_variable_name(registered_var):
-            return "Task registers a variable that violates variable naming standards"
+            results.append(
+                self.create_matcherror(
+                    filename=file,
+                    linenumber=task[LINE_NUMBER_KEY],
+                    message=f"Task registers a variable that violates variable naming standards: {registered_var}",
+                    tag=f"var-naming[{registered_var}]",
+                )
+            )
 
-        return False
+        return results
 
     def matchyaml(self, file: Lintable) -> list[MatchError]:
         """Return matches for variables defined in vars files."""
@@ -190,28 +185,32 @@ class VariableNamingRule(AnsibleLintRule):
 
 # testing code to be loaded only with pytest or when executed the rule file
 if "pytest" in sys.modules:
-
     import pytest
 
-    from ansiblelint.testing import RunFromText  # pylint: disable=ungrouped-imports
+    from ansiblelint.testing import (  # pylint: disable=ungrouped-imports
+        RunFromText,
+        run_ansible_lint,
+    )
 
     @pytest.mark.parametrize(
-        "rule_runner", (VariableNamingRule,), indirect=["rule_runner"]
+        ("file", "expected"),
+        (
+            pytest.param("examples/playbooks/rule-var-naming-fail.yml", 7, id="0"),
+            pytest.param("examples/Taskfile.yml", 0, id="1"),
+        ),
     )
-    def test_invalid_var_name_playbook(rule_runner: RunFromText) -> None:
+    def test_invalid_var_name_playbook(file: str, expected: int) -> None:
         """Test rule matches."""
-        results = rule_runner.run_playbook(FAIL_PLAY)
-        assert len(results) == 4
+        rules = RulesCollection(options=options)
+        rules.register(VariableNamingRule())
+        results = Runner(Lintable(file), rules=rules).run()
+        # results = rule_runner.run()
+        assert len(results) == expected
         for result in results:
             assert result.rule.id == VariableNamingRule.id
-
-        # list unexpected error lines or non-matching error lines
-        expected_error_lines = [4, 8, 18, 20]
-        lines = [i.linenumber for i in results]
-        error_lines_difference = list(
-            set(expected_error_lines).symmetric_difference(set(lines))
-        )
-        assert len(error_lines_difference) == 0
+        # We are not checking line numbers because they can vary between
+        # different versions of ruamel.yaml (and depending on presence/absence
+        # of its c-extension)
 
     @pytest.mark.parametrize(
         "rule_runner", (VariableNamingRule,), indirect=["rule_runner"]
@@ -230,3 +229,14 @@ if "pytest" in sys.modules:
             set(expected_error_lines).symmetric_difference(set(lines))
         )
         assert len(error_lines_difference) == 0
+
+    def test_var_naming_with_pattern() -> None:
+        """Test rule matches."""
+        role_path = "examples/roles/var_naming_pattern/tasks/main.yml"
+        conf_path = "examples/roles/var_naming_pattern/.ansible-lint"
+        result = run_ansible_lint(
+            f"--config-file={conf_path}",
+            role_path,
+        )
+        assert result.returncode == SUCCESS_RC
+        assert "var-naming" not in result.stdout
